﻿namespace TabsStudioNavigator
{
    class Commands
    {
        public Commands(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_, CommandsImpl commandsImpl_)
        {
            engine = engine_;
            dte = dte_;
            commandsImpl = commandsImpl_;
            if (!new TabsStudioExt.Util.VsVersion(dte).IsExtension())
                RegisterCommands();
        }

        public void QueryStatus(string CmdName, EnvDTE.vsCommandStatusTextWanted NeededText,
                ref EnvDTE.vsCommandStatus StatusOption, ref System.Object CommandText)
        {
            if (NeededText != EnvDTE.vsCommandStatusTextWanted.vsCommandStatusTextWantedNone || CmdName == null)
                return;

            if (CmdName == "TabsStudio.Connect.NavigateToNextTab" ||
                CmdName == "TabsStudio.Connect.NavigateToPreviousTab" ||
                CmdName == "TabsStudio.Connect.NavigateToLastTab" ||
                CmdName == "TabsStudio.Connect.NavigateToNextWindow" ||
                CmdName == "TabsStudio.Connect.NavigateToPreviousWindow" ||
                CmdName == "TabsStudio.Connect.NavigateToNextGroup" ||
                CmdName == "TabsStudio.Connect.NavigateToPreviousGroup" ||
                CmdName == "TabsStudio.Connect.MoveTabLeft" ||
                CmdName == "TabsStudio.Connect.MoveTabRight" ||
                CmdName.StartsWith(tabCommandPrefix))
            {
                StatusOption = (EnvDTE.vsCommandStatus)SupportedAndEnabled();
            }
        }

        public void Exec(string CmdName, EnvDTE.vsCommandExecOption ExecuteOption, ref System.Object VariantIn,
            ref System.Object VariantOut, ref bool Handled)
        {
            Handled = false;
            if (ExecuteOption != EnvDTE.vsCommandExecOption.vsCommandExecOptionDoDefault || CmdName == null)
                return;

            if (CmdName == "TabsStudio.Connect.NavigateToNextTab")
            {
                commandsImpl.NavigateToNextTab();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.NavigateToPreviousTab")
            {
                commandsImpl.NavigateToPreviousTab();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.NavigateToLastTab")
            {
                commandsImpl.NavigateToLastTab();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.NavigateToNextWindow")
            {
                commandsImpl.NavigateToNextWindow();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.NavigateToPreviousWindow")
            {
                commandsImpl.NavigateToPreviousWindow();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.NavigateToNextGroup")
            {
                commandsImpl.NavigateToNextGroup();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.NavigateToPreviousGroup")
            {
                commandsImpl.NavigateToPreviousGroup();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.MoveTabLeft")
            {
                commandsImpl.MoveTabLeft();
                Handled = true;
            }
            if (CmdName == "TabsStudio.Connect.MoveTabRight")
            {
                commandsImpl.MoveTabRight();
                Handled = true;
            }
            if (CmdName.StartsWith(tabCommandPrefix))
            {
                System.UInt16 i;
                if (System.UInt16.TryParse(CmdName.Substring(tabCommandPrefix.Length), out i) && i >= 1)
                    commandsImpl.NavigateToTab(i);
                Handled = true;
            }
        }

        private void RegisterCommands()
        {
            EnvDTE80.Commands2 commands = dte.Commands as EnvDTE80.Commands2;
            if (commands == null)
                return;

            RegisterCommand(commands, engine.GetAddIn(), "NavigateToNextTab", "Navigate to Next Tab",
                "Activate Next Tab");
            RegisterCommand(commands, engine.GetAddIn(), "NavigateToPreviousTab", "Navigate to Previous Tab",
                "Activate Previous Tab");
            RegisterCommand(commands, engine.GetAddIn(), "NavigateToLastTab", "Navigate to Last Tab",
                "Activate Last Tab");
            RegisterCommand(commands, engine.GetAddIn(), "NavigateToNextWindow", "Navigate to Next Window",
                "Activate Next Window");
            RegisterCommand(commands, engine.GetAddIn(), "NavigateToPreviousWindow", "Navigate to Previous Window",
                "Activate Previous Window");
            RegisterCommand(commands, engine.GetAddIn(), "NavigateToNextGroup", "Navigate to Next Tab Group",
                "Activate Next Tab Group");
            RegisterCommand(commands, engine.GetAddIn(), "NavigateToPreviousGroup", "Navigate to Previous Tab Group",
                "Activate Previous Tab Group");
            RegisterCommand(commands, engine.GetAddIn(), "MoveTabLeft", "Move Tab Left",
                "Move Tab Left");
            RegisterCommand(commands, engine.GetAddIn(), "MoveTabRight", "Move Tab Right",
                "Move Tab Right");

            for (int i = 1; i <= 20; ++i)
                RegisterCommand(commands, engine.GetAddIn(), "NavigateToTab" + i, "Navigate to Tab #" + i,
                    "Activate Tab #" + i);

            //commands.Item("TabsStudio.Connect.NavigateToNextTab", 0).Delete();
        }

        private void RegisterCommand(EnvDTE80.Commands2 commands, EnvDTE.AddIn addin,
            string name, string buttonText, string tooltip)
        {
            try
            {
                object[] contextGUIDS = new object[] { };
                commands.AddNamedCommand2(addin, name, buttonText, tooltip,
                    true, 36, ref contextGUIDS,
                    (int)SupportedAndEnabled(),
                    (int)EnvDTE80.vsCommandStyle.vsCommandStylePictAndText,
                    EnvDTE80.vsCommandControlType.vsCommandControlTypeButton);
            }
            catch (System.ArgumentException)
            {
                // Command already exists.
            }
        }

        private int SupportedAndEnabled()
        {
            return (int)EnvDTE.vsCommandStatus.vsCommandStatusSupported +
                (int)EnvDTE.vsCommandStatus.vsCommandStatusEnabled;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
        private CommandsImpl commandsImpl;
        private readonly string tabCommandPrefix = "TabsStudio.Connect.NavigateToTab";
    }
}
