﻿namespace TabsStudioNavigator
{
    class Tab
    {
        public Tab(TabsStudioExt.Tab tab_, bool useActivateTabFocusWorkaround_)
        {
            tab = tab_;
            useActivateTabFocusWorkaround = useActivateTabFocusWorkaround_;
        }

        public Extension GetActiveExtension()
        {
            foreach (TabsStudioExt.TabExtension tabExtension in tab.TabExtensions)
            {
                if (tabExtension.IsExtensionActive)
                    return new Extension(tabExtension);
            }
            return null;
        }

        public void Activate()
        {
            ActivateExtension(GetActiveExtension());
        }

        public void ActivateExtension(Extension ext)
        {
            if (useActivateTabFocusWorkaround)
            {
                tab.TabItem.Focusable = true;
                tab.TabItem.Focus();
                tab.TabItem.Focusable = false;
            }
            ext.Activate();
        }

        public uint Count()
        {
            return (uint)tab.TabExtensions.Count;
        }

        // REQUIRE(i >= 0 && i < Count())
        public Extension GetExtension(uint i)
        {
            if (Count() == 1)
                return new Extension(tab.TabExtensions[0]);

            return FindExtension(GetExtensionLabels()[(int)i]);
        }

        public bool IsEqual(System.Windows.Controls.TabItem item)
        {
            return tab.TabItem == item;
        }

        public uint GetExtensionIndex(Extension ext)
        {
            if (tab.TabExtensions.Count == 1)
                return 0;

            uint result = 0;
            foreach (System.Windows.Controls.Label label in GetExtensionLabels())
            {
                if (ext.IsEqual(label))
                    break;
                ++result;
            }
            return result;
        }

        public System.Windows.Controls.TabItem GetTabItem()
        {
            return tab.TabItem;
        }

        // REQUIRE(tab.TabExtensions.Count > 1)
        private System.Collections.Generic.List<System.Windows.Controls.Label> GetExtensionLabels()
        {
            System.Collections.Generic.List<System.Windows.Controls.Label> result =
                new System.Collections.Generic.List<System.Windows.Controls.Label>();
            System.Windows.Controls.DockPanel dock = tab.TabItem.Header as System.Windows.Controls.DockPanel;
            if (dock != null && dock.Children.Count >= 2)
            {
                for (int i = 0; i < tab.TabExtensions.Count; ++i)
                {
                    result.Add((dock.Children[i + 1] as System.Windows.Controls.StackPanel).Children[0] as
                        System.Windows.Controls.Label);
                }
            }
            return result;
        }

        private Extension FindExtension(System.Windows.Controls.Label label)
        {
            foreach (TabsStudioExt.TabExtension ext in tab.TabExtensions)
            {
                Extension result = new Extension(ext);
                if (result.IsEqual(label))
                    return result;
            }
            return null;
        }

        private TabsStudioExt.Tab tab;
        private bool useActivateTabFocusWorkaround;
    }
}
