﻿namespace TabsStudioNavigator
{
    class TabNumbers
    {
        public TabNumbers(TabsStudioExt.ITabsStudioEngine engine, Tabs tabs)
        {
            this.engine = engine;
            this.tabsHelper = tabs;

            engine.TabsCreated += OnTabsCreated;
            engine.TabsDestroyed += OnTabsDestroyed;
            engine.TabCreated += OnTabCreated;

            delayedLayoutUpdateTimer = new System.Windows.Threading.DispatcherTimer();
            delayedLayoutUpdateTimer.Interval = System.TimeSpan.FromMilliseconds(200);
            delayedLayoutUpdateTimer.Tick += OnDelayedLayoutUpdateTimerTick;
        }

        public void Show(bool show)
        {
            this.show = show;
            delayedLayoutUpdateTimer.Stop();

            if (show)
                ShowNumbers();
            else
                HideNumbers();
        }

        public void Close()
        {
            delayedLayoutUpdateTimer.Stop();

            engine.TabsCreated -= OnTabsCreated;
            engine.TabsDestroyed -= OnTabsDestroyed;
            engine.TabCreated -= OnTabCreated;
        }

        private void OnTabsCreated(object sender, TabsStudioExt.TabsEventArgs e)
        {
            e.Tabs.TabPanel2.LayoutUpdated += OnLayoutUpdated;
        }

        private void OnTabsDestroyed(object sender, TabsStudioExt.TabsEventArgs e)
        {
            e.Tabs.TabPanel2.LayoutUpdated -= OnLayoutUpdated;
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (show)
                ShowNumbers(e.Tabs);
        }

        private void OnLayoutUpdated(object sender, System.EventArgs e)
        {
            if (show)
            {
                delayedLayoutUpdateTimer.Stop();
                delayedLayoutUpdateTimer.Start();
            }
        }

        private void OnDelayedLayoutUpdateTimerTick(System.Object o, System.EventArgs e)
        {
            delayedLayoutUpdateTimer.Stop();
            if (show)
                ShowNumbers();
        }

        private void ShowNumbers()
        {
            foreach (TabsStudioExt.Tabs tabs in engine.GetTabs())
                ShowNumbers(tabs);
        }

        private void ShowNumbers(TabsStudioExt.Tabs tabs)
        {
            var visibleTabs = tabsHelper.GetVisibleTabs(tabs);
            for (int i = 0; i < visibleTabs.Count; ++i)
                ShowNumber(FindTab(tabs, visibleTabs[i]), i + 1);
        }

        private void HideNumbers()
        {
            foreach (TabsStudioExt.Tabs tabs in engine.GetTabs())
                foreach (TabsStudioExt.Tab tab in tabs.TabList)
                    HideNumber(tab);
        }

        private TabsStudioExt.Tab FindTab(TabsStudioExt.Tabs tabs, System.Windows.Controls.TabItem item)
        {
            foreach (TabsStudioExt.Tab tab in tabs.TabList)
            {
                if (tab.TabItem == item)
                    return tab;
            }
            return null;
        }

        private void ShowNumber(TabsStudioExt.Tab tab, int number)
        {
            string replace = number.ToString() + "  $_";
            foreach (TabsStudioExt.TitleTransform transform in tab.TitleManager.Transforms)
            {
                if (transform.Id == tabNumberTag)
                {
                    if (string.Equals(transform.ReplacementRegex, replace))
                        return;

                    tab.TitleManager.Transforms.Remove(transform);
                    break;
                }
            }
            tab.TitleManager.Transforms.Add(new TabsStudioExt.TitleTransform("^.*$", replace, tabNumberTag));
            tab.TitleManager.ApplyTransforms();
        }

        private void HideNumber(TabsStudioExt.Tab tab)
        {
            foreach (TabsStudioExt.TitleTransform transform in tab.TitleManager.Transforms)
            {
                if (transform.Id == tabNumberTag)
                {
                    tab.TitleManager.Transforms.Remove(transform);
                    tab.TitleManager.ApplyTransforms();
                    return;
                }
            }
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private Tabs tabsHelper;
        private bool show;
        private System.Windows.Threading.DispatcherTimer delayedLayoutUpdateTimer;
        private static string tabNumberTag = "[NavigatorTabNumber]";
    }
}
