﻿namespace TabsStudioNavigator
{
    class Tabs
    {
        public Tabs(TabsStudioExt.ITabsStudioEngine engine_, bool useActivateTabFocusWorkaround_)
        {
            engine = engine_;
            useActivateTabFocusWorkaround = useActivateTabFocusWorkaround_;
            isPreviewTabProperty = new TabsStudioExt.Util.IsPreviewTabProperty();
        }

        public uint Count()
        {
            TabsStudioExt.Tabs tabs = FindSelectedTabs();
            if (tabs == null)
                return 0;

            return (uint)GetVisibleTabs().Count;
        }

        public uint GroupCount()
        {
            return (uint)GetVisibleGroups().Count;
        }

        // REQUIRE(i >= 0 && i < Count())
        public Tab GetTab(uint i)
        {
            return FindTab(GetVisibleTabs()[(int)i]);
        }

        // REQUIRE(i >= 0 && i < GroupCount())
        public Group GetGroup(uint i)
        {
            return GetVisibleGroups()[(int)i];
        }

        // REQUIRE(Count() > 0)
        public Tab GetSelectedTab()
        {
            Group group = GetSelectedGroup();
            if (group == null)
                return null;

            return group.GetSelectedTab();
        }

        public Group GetSelectedGroup()
        {
            TabsStudioExt.Tabs tabs = FindSelectedTabs();
            if (tabs == null)
                return null;

            return new Group(tabs, useActivateTabFocusWorkaround);
        }

        public uint GetTabIndex(Tab tab)
        {
            uint result = 0;
            foreach (System.Windows.Controls.TabItem item in GetVisibleTabs())
            {
                if (tab.IsEqual(item))
                    break;
                ++result;
            }
            return result;
        }

        public uint GetGroupIndex(Group group)
        {
            uint result = 0;
            foreach (Group item in GetVisibleGroups())
            {
                if (group.IsEqual(item))
                    break;
                ++result;
            }
            return result;
        }

        public void MoveTab(Tab source, Tab target)
        {
            System.Windows.Controls.Panel panel = FindSelectedTabs().TabPanel2;
            int target_index = panel.Children.IndexOf(target.GetTabItem());
            panel.Children.Remove(source.GetTabItem());
            panel.Children.Insert(target_index, source.GetTabItem());
        }

        public System.Collections.Generic.List<System.Windows.Controls.TabItem> GetVisibleTabs(TabsStudioExt.Tabs tabs)
        {
            System.Collections.Generic.List<System.Windows.Controls.TabItem> result =
                new System.Collections.Generic.List<System.Windows.Controls.TabItem>();
            System.Windows.Controls.TabItem previewTab = null;
            foreach (System.Windows.UIElement i in tabs.TabPanel2.Children)
            {
                System.Windows.Controls.TabItem item = i as System.Windows.Controls.TabItem;
                if (item.Visibility == System.Windows.Visibility.Visible)
                {
                    if (!isPreviewTabProperty.IsPreviewTab(item))
                        result.Add(item);
                    else
                        previewTab = item;
                }
            }
            if (previewTab != null)
                result.Add(previewTab);
            return result;
        }

        private TabsStudioExt.Tabs FindSelectedTabs()
        {
            foreach (TabsStudioExt.Tabs tabs in engine.ReturnTabs())
            {
                if (tabs.IsGroupSelected)
                    return tabs;
            }
            return null;
        }

        private System.Collections.Generic.List<System.Windows.Controls.TabItem> GetVisibleTabs()
        {
            return GetVisibleTabs(FindSelectedTabs());
        }

        private System.Collections.Generic.List<Group> GetVisibleGroups()
        {
            System.Collections.Generic.List<Group> result = new System.Collections.Generic.List<Group>();
            foreach (TabsStudioExt.Tabs tabs in engine.ReturnTabs())
                result.Add(new Group(tabs, useActivateTabFocusWorkaround));
            result.Sort(
                    delegate (Group lhsg, Group rhsg)
                    {
                        System.Windows.Point lhs = lhsg.GetPosition();
                        System.Windows.Point rhs = rhsg.GetPosition();
                        int cmpX = lhs.X.CompareTo(rhs.X);
                        if (cmpX != 0)
                            return cmpX;
                        return lhs.Y.CompareTo(rhs.Y);
                    });
            return result;
        }

        private Tab FindTab(System.Windows.Controls.TabItem item)
        {
            foreach (TabsStudioExt.Tab tab in FindSelectedTabs().TabList)
            {
                Tab result = new Tab(tab, useActivateTabFocusWorkaround);
                if (result.IsEqual(item))
                    return result;
            }
            return null;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private bool useActivateTabFocusWorkaround;
        private TabsStudioExt.Util.IsPreviewTabProperty isPreviewTabProperty;
    }
}
